;; Generated package description from excorporate.el  -*- no-byte-compile: t -*-
(define-package "excorporate" "0.9.2" "Exchange Web Services (EWS) integration" '((emacs "24.1") (fsm "0.2.1") (soap-client "3.2.0") (url-http-ntlm "2.0.4") (nadvice "0.3")) :keywords '("calendar") :authors '(("Thomas Fitzsimmons" . "fitzsim@fitzsim.org")) :maintainer '("Thomas Fitzsimmons" . "fitzsim@fitzsim.org") :url "https://www.fitzsim.org/blog/")
